#!/bin/bash
_self_bin_name="$0"
function where_is_him () {
    SOURCE="$1"
    while [ -h "$SOURCE" ]; do # resolve $SOURCE until the file is no longer a symlink
        DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
        SOURCE="$(readlink "$SOURCE")"
        [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE" # if $SOURCE was a relative symlink, we need to resolve it relative to the path where the symlink file was located
    done
    DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
    echo -n "$DIR"
}

function where_am_i () {
    _my_path=`type -p ${_self_bin_name}`
    [[ "$_my_path" = "" ]] && where_is_him "$_self_bin_name" || where_is_him "$_my_path"
}

! source "$(where_am_i)/antidote.config.sh" && echo "Failed to source config file" && exit 2

##############################################################################3
function upload_to_smb_share () {
    # returns smb_url from stdout
    release_ver="$1"
    local_wf_dir="$2"

    # The workflow builder should copy all xaml file to there. 
    find "$local_wf_dir" -name '*.pdb' -exec rm -f '{}' ';'
    cmd="deltree /antidote-cis/$smb_namespace/$release_ver;mkdir /antidote-cis;mkdir /antidote-cis/$smb_namespace;mkdir /antidote-cis/$smb_namespace/$release_ver;mask \"\";recurse ON;prompt OFF;cd /antidote-cis/$smb_namespace/$release_ver;lcd $local_wf_dir;mput *"
    
    smbclient "$smb_provider" --user "$smb_username%$smb_password" -c "$cmd" 1>&2 || return 2
    smb_url="$smb_provider/antidote-cis/$smb_namespace/$release_ver"
    echo "$smb_url"
}

# CIS upload pkg
function cis_upload_pkg () {
    workflow_name="$1"
    workflow_name="$1"
    release_ver="$2"
    pkg_path="$3"
    smb_url=`upload_to_smb_share "$release_ver" "$pkg_path"` || return $?
    echo "Submitting request to upload package $cis_namespace.$workflow_name:$release_ver, using smb:$smb_url..."
    curl --data '[{"Cloud":"Public","PackageType":"'"$cis_namespace.$workflow_name"'","Version":"'"$release_ver"'","PackageId":"00000000-0000-0000-0000-000000000000","ActionType":501,"Notes":"antidote-auto-upload","ActionArgs":{"PackageSourcePath":"'"$smb_url"'","ScriptParameters":""}}]' "https://beta-cps.trafficmanager.net/cis.svc/Public/$cis_namespace.$workflow_name/Packages/Request" --cookie "$cis_cookie" -H 'content-type: application/json' --max-time 30
    return $?
}

function try_grep_pkgid () {
    workflow_name="$1"
    release_ver="$2"
    resp=`curl -s "https://beta-cps.trafficmanager.net/cis.svc/Public/$cis_namespace.$workflow_name/Packages?null" --cookie "$cis_cookie" --max-time 30` || return $?
    [[ "$resp" = "" ]] && echo "Invalid api response. Is cookie expired or not correctly set?" && return 2
    ! echo "$resp" | json2table | grep -F " $release_ver " > /dev/null && echo "Error: release_ver $release_ver doesn't exist at all" && return 2

    # output pkgid
    pkgid=`echo "$resp" | json2table | grep -v 00000000-0000-0000-0000-000000000000 | grep -F " $release_ver " | cut -d '|' -f 5 | tr -d ' '`
    [[ "$pkgid" = "" ]] && return 2
    echo "$pkgid"
}


function cis_release_pkg () {
    workflow_name="$1"
    release_ver="$2"
    pkg_path="$3"
    echo "Release package $cis_namespace.$workflow_name:$release_ver..."
    pkg_id=`try_grep_pkgid "$workflow_name" "$release_ver"` || return $?
    curl --data '[{"Cloud":"Public","PackageType":"'"$cis_namespace.$workflow_name"'","Version":"'"$release_ver"'","PackageId":"'"$pkg_id"'","ActionType":505,"Notes":"antidote-auto-release","ActionArgs":{"PackageSourcePath":null,"ScriptParameters":""}}]' "https://beta-cps.trafficmanager.net/cis.svc/Public/$cis_namespace.$workflow_name/Packages/Request" --cookie "$cis_cookie" -H 'content-type: application/json' --max-time 30
    return $?
}

# CIS show pkg list
# GET https://beta-cps.trafficmanager.net/cis.svc/Public/M365FleetAGCTest.E2EPOCBuildPortalWorkflow/Packages?null

# CIS wait upload pkg success
function cis_wait_for_upload () {
    workflow_name="$1"
    release_ver="$2"
    echo "Waiting for CIS to create package..."
    while true; do
        try_grep_pkgid "$cis_namespace.$workflow_name" "$release_ver" && break # success
        echo -n .
        sleep 5
    done
}


function cis_run_job () {
    workflow_name="$1"
    release_ver="$2"
    pkg_id=`try_grep_pkgid "$workflow_name" "$release_ver"` || return $?
    job_id=`curl "https://beta-cps.trafficmanager.net/cis.svc/Public/$cis_namespace/GenericJob?workflowDefinitionName=$workflow_name" -H 'content-type: application/json' --cookie "$cis_cookie" --max-time 30 --data '{"JobType":"'"$cis_namespace"'","Workflow":"'"$workflow_name"'","ServiceHostCapability":"AzureJB","ServiceHostLocation":"Any","RuntimePackageId":"4dd32e8b-6bc7-478e-a1ab-48fcdc1bf3d3","PackageId":"'"$pkg_id"'","DisplayName":"Antidote-autosubmitted-WF","WorkflowParameters":{"Alias":"bensl","Date":"03/26/2021","MdmId":"202103260008"},"RuntimeSettings":{"RunEnvironment":"CISBeta"},"WorkflowSettings":{"RuntimeSettings":{"RunEnvironment":"CISBeta"}}}'` || return $?
    
    echo "Waiting for CIS to create job... (job_id=$job_id)"
    niddle_should_wait='"DisplayStatus":"NotStarted"'
    while true; do
        curl "https://beta-cps.trafficmanager.net/cis.svc/Public/$cis_namespace/GenericJob/$job_id/GetJobHierarchy?null" -H 'content-type: application/json' --cookie "$cis_cookie" --max-time 30 | grep "$niddle_should_wait" && break
        echo -n .
        sleep 5
    done
    
    echo "Finally: Starting job..."
    curl "https://beta-cps.trafficmanager.net/cis.svc/Public/$cis_namespace/GenericJob/$job_id/Start/" -X 'PUT' -H 'content-type: application/json' -H 'accept: application/json, text/javascript, */*; q=0.01' --cookie "$cis_cookie" --data-raw '{"Notes":"antidote-autostart"}'

    return $?
}



antidote_version="0.8.1a"
subcmd="$1"
shift

case "$subcmd" in
    push)
        local_wf_dir="$1"
        workflow_name="$2"
        [[ "$workflow_name" = "" ]] && workflow_name="$cis_default_workflow_name"
        release_ver="$3"
        [[ "$release_ver" = "" ]] && release_ver="ricin.$RANDOM$RANDOM"
        echo "$release_ver" > /tmp/.antidote-cis.latest-ver
        ;;
    cloudrun)
        workflow_name="$1"
        [[ "$workflow_name" = "" ]] && workflow_name="$cis_default_workflow_name"
        release_ver="$2"
        [[ "$release_ver" = "" ]] && release_ver=`cat /tmp/.antidote-cis.latest-ver` || { echo "Error: Unable to determine" ; exit 1 ; }
        ;;
    *)
        echo "
Antidote-CIS version $antidote_version, maintained by Recolic Keghart <root@recolic.net> or Bensong Liu <bensl@microsoft.com> (the same person!)

usage: $0 push path/to/your/workflow/dir [BootstrapCAWorkflow] [v4.1.1]
usage: $0 cloudrun [BootstrapCAWorkflow] [v4.1.1]
If you omit the version number, 'push' would generate a random verion number, and 'cloudrun' would use the version number in your last push. 
If you omit the workflow name, antidote would use the cis_default_workflow_name in antidote.config.sh"
        exit 1
esac


cis_upload_pkg "$workflow_name" "$release_ver" "$local_wf_dir" &&
cis_wait_for_upload "$workflow_name" "$release_ver" &&
cis_release_pkg "$workflow_name" "$release_ver" &&
cis_run_job "$workflow_name" "$release_ver"

#echo "Submitting request to run task..."
#sleep 5
#echo "Done. result_url = https://beta-cps.trafficmanager.net/Public/M365FleetAGCTest/JobDetails/2517857189480004267_6c109e40-4cba-44c0-bfd5-f60e113489eb"


